import functools
import contextlib

from typing import Dict, Callable

from service_now import client

ConnectionInfo = Dict[str, str]


@contextlib.contextmanager
def _api_client(access_details: ConnectionInfo):
    missing_keys = []
    for key in {"address", "access_token"}:
        if key not in access_details:
            missing_keys.append(key)
    if missing_keys:
        raise Exception(
            "ServiceNow API functions require both an address and access_token in the "
            f"connection details. The following keys are missing: {missing_keys}."
        )
    yield client.initialize(**access_details)


def _api_function(func: Callable):
    @functools.wraps(func)
    def _inner(access_details: ConnectionInfo, *args, **kwargs):
        with _api_client(access_details) as api_client:
            return func(api_client, *args, **kwargs)

    return _inner
